<?php
/**
 * Sets up the plugin's hooks.
 *
 * @since 0.1.0
 * @package ethpress
 */

namespace losnappas\Ethpress;

defined( 'ABSPATH' ) || die;

use losnappas\Ethpress\Front;
use losnappas\Ethpress\Login;
use losnappas\Ethpress\Upgrade;

/**
 * Static functions only.
 *
 * @since 0.1.0
 */
class Plugin {

	/**
	 * Table names. Don't forget to base_prefix. Base_ because it's user related and does not have separate tables for every site.
	 *
	 * @since 0.1.0
	 *
	 * @var array
	 */
	public static $tables = [
		'addresses' => 'ethpress_addresses',
	];

	/**
	 * This contains hook and filter assignments, etc.
	 *
	 * @since 0.1.0
	 */
	public static function attach_hooks() {
		add_action( 'plugins_loaded', [ __CLASS__, 'load_plugin_textdomain' ] );

		add_action( 'login_form', [ ETHPRESS_NS . '\Front', 'login_form' ] );
		add_action( 'login_enqueue_scripts', [ __CLASS__, 'register_scripts' ], 9 );
		add_action( 'login_enqueue_scripts', [ __CLASS__, 'login_enqueue_scripts' ] );

		add_action( 'wp_ajax_nopriv_ethpress_log_in', [ ETHPRESS_NS . '\Login', 'verify_login' ] );
		add_action( 'wp_ajax_nopriv_ethpress_get_message', [ ETHPRESS_NS . '\Login', 'get_message' ] );
		add_action( 'wp_ajax_ethpress_get_message', [ __CLASS__, 'logged_in_user_error' ] );
		add_action( 'deleted_user', [ ETHPRESS_NS . '\Login', 'destroy' ] );

		add_action( 'ethpress_login', [ ETHPRESS_NS . '\Login', 'attach_user_to_blog' ] );
	}

	/**
	 * Registers the scripts so they're available in login_enqueue_scripts and wp_enqueue_scripts.
	 *
	 * @since 0.1.1
	 */
	public static function register_scripts() {
		// Styles for just the button.
		wp_register_style(
			'ethpress-login',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/css/login.css',
			[],
			'1.0.0'
		);
		// Includes styles for the info popup.
		wp_register_style(
			'ethpress-login-front',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/css/login-front.css',
			[ 'dashicons', 'ethpress-login' ],
			'1.0.0'
		);
		// Provides ethpress.metamask.connect. Loaded in header for easier access.
		wp_register_script(
			'ethpress-login',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/js/login.min.js',
			[],
			'1.0.0',
			false
		);
		// Adds event listener to button.
		wp_register_script(
			'ethpress-login-front',
			plugin_dir_url( ETHPRESS_FILE ) . 'public/js/login-front.min.js',
			[ 'ethpress-login' ],
			'1.0.0',
			true
		);

		wp_localize_script(
			'ethpress-login',
			'ethpressLoginWP',
			[
				'ajaxUrl'       => admin_url( 'admin-ajax.php' ),
				'loginNonce'    => wp_create_nonce( 'ethpress_log_in' ),
				'getNonceNonce' => wp_create_nonce( 'ethpress_get_message' ),
				'strings'       => [
					'waitPermission'   => esc_html__( 'Waiting for permission', 'ethpress' ),
					'waitSignature'    => esc_html__( 'Waiting for signature', 'ethpress' ),
					'verifySignature'  => esc_html__( 'Verifying signature', 'ethpress' ),
					'notInstalled'     => esc_html__( 'MetaMask is not installed', 'ethpress' ),
					'aborted'          => esc_html__( 'Login aborted', 'ethpress' ),
					'unknownError'     => esc_html__( 'Unknown error', 'ethpress' ),
					'fetchingDocument' => esc_html__( 'Fetching document', 'ethpress' ),
				],
			]
		);
	}

	/**
	 * Attached to hook.
	 *
	 * @since 0.1.0
	 */
	public static function login_enqueue_scripts() {
		// Only enqueue the -front ones. They'll pull the other ones. This way it's easier to remove them.
		wp_enqueue_style( 'ethpress-login-front' );
		wp_enqueue_script( 'ethpress-login-front' );
	}

	/**
	 * Sends an error message if trying to log in with MetaMask while logged in.
	 *
	 * @since 0.1.0
	 */
	public static function logged_in_user_error() {
		wp_send_json_error( esc_html__( 'Log out first', 'ethpress' ) );
	}

	/**
	 * Load translation files.
	 *
	 * @since 0.1.0
	 */
	public static function load_plugin_textdomain() {
		$path  = dirname( plugin_basename( ETHPRESS_FILE ) );
		$path .= '/languages';
		load_plugin_textdomain( 'ethpress', false, $path );
	}

	/**
	 * Is called from uninstall.php
	 *
	 * TODO: add a conditional ETHPRESS_REMOVE_USERS and remove all user accounts if true?
	 *
	 * @since 0.1.0
	 */
	public static function uninstall() {
		defined( 'WP_UNINSTALL_PLUGIN' ) || die;

		global $wpdb;

		$table = $wpdb->base_prefix . self::$tables['addresses'];

		// phpcs:ignore -- No cache, schema change, no prepare.
		$wpdb->query( "DROP TABLE IF EXISTS {$table}" );
		delete_option( 'ethpress' );
	}

	/**
	 * Creates database tables on plugin activation.
	 *
	 * @since 0.1.0
	 */
	public static function activate() {
		global $wpdb;

		$table      = $wpdb->base_prefix . self::$tables['addresses'];
		$db_version = '1.1';
		$api_url    = 'https://verify-eth-signature.herokuapp.com/ethpress';
		$opts       = [
			'db_version' => $db_version,
			'api_url'    => $api_url,
		];
		/**
		 * Ethpress' settings.
		 *
		 * @since 0.1.0
		 *
		 * Settings: db_version (internal), api_url: see Signature.php.
		 */
		add_option( 'ethpress', $opts );

		require_once ABSPATH . 'wp-admin/includes/upgrade.php';

		// No multisite stuff here because user table only exists once.
		$charset_collate = '';
		if ( $wpdb->has_cap( 'collation' ) ) {
			$charset_collate = $wpdb->get_charset_collate();
		}
		$sql = "
CREATE TABLE {$table} (
  id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
  name varchar(191) NOT NULL,
  user_id BIGINT UNSIGNED NOT NULL,
  coin varchar(15) DEFAULT NULL,
  date datetime DEFAULT '2000-01-01 00:00:00',
  modified datetime DEFAULT '2000-01-01 00:00:00',
  PRIMARY KEY  (id),
  UNIQUE KEY name (name),
  KEY user_id (user_id)
) $charset_collate;";
		\dbDelta( $sql );

		Upgrade::handle_upgrades();
	}
}
