'use strict';

/**
 * Handles login interaction with MetaMask.
 */
var ethpress = ( function( web3, ethereum, ethpressLoginWP ) {
	var _strings = ethpressLoginWP.strings,
		_that,
		_address;

	/**
	 * Get nonce for current wallet, or register. Call window.ethereum.enable before calling this.
	 */
	var getLoginMessage = function( addresses ) {
		var data = new URLSearchParams();

		// Using web3.eth.coinbase is unreliable.
		_address = addresses[0];
		data.append( 'action', 'ethpress_get_message' );
		data.append( '_ajax_nonce', ethpressLoginWP.getNonceNonce );
		data.append( 'coinbase', _address );
		return fetch( ethpressLoginWP.ajaxUrl + '?' + data.toString() );
	};

	/**
	 * Sets button text.
	 *
	 * @param string statusText Short text describing status.
	 * @param bool   disable    True if button should be disabled, false if enabled, and null if no change.
	 */
	var setStatus = function( statusText, disable ) {

		if ( _that instanceof Element ) {
			if ( true === disable ) {
				_that.disabled = true;
			} else if ( false === disable ) {
				_that.disabled = false;
			}

			_that.innerText = statusText;
		}
	};

	var metamask = {

		/**
		 * Register and log in.
		 */
		connect: function() {
			if ( ! web3 || ! web3.personal || ! ethereum || ! ethereum.isMetaMask ) {
				setStatus( _strings.notInstalled, true );
				return;
			}

			// Status texts will be displayed on the button element itself.
			_that = this;

			// Disable button during login procedures.
			setStatus( _strings.waitPermission, true );

			ethereum.enable()
			.then( function( addresses ) {
				setStatus( _strings.fetchingDocument );
				return addresses;
			})
			.then( getLoginMessage )
			.then( function( res ) {
				if ( res.ok ) {
					return res.json();
				}
				throw new Error( res.statusText );
			})
			.then( function( result ) {
				if ( ! result.success ) {
					throw new Error( result.data );
				}

				setStatus( _strings.waitSignature );

				// Message was signed, now verify it.
				web3.personal.sign( web3.fromUtf8( result.data ), _address, metamask.verify );
			})
			.catch( function( err ) {

				// Cancelled at "Connect" screen.
				if ( null != err.code && -32603 === err.code ) {
					err.message = _strings.aborted;
				}

				// Enable button on error so user can retry.
				setStatus( err.message, false );
			});
		},

		verify: function( err, signedMessage ) {
			var data = new FormData();

			if ( null != err ) {
				setStatus( _strings.aborted, false );
				return;
			}

			if ( 'string' !== typeof signedMessage ) {
				setStatus( _strings.unknownError, false );
				return;
			}

			data.append( 'signature', signedMessage );
			data.append( '_ajax_nonce', ethpressLoginWP.loginNonce );
			data.append( 'action', 'ethpress_log_in' );
			data.append( 'coinbase', web3.eth.coinbase );
			data.append( 'redirect_to', new URLSearchParams( window.location.search ).get( 'redirect_to' ) || '' );

			setStatus( _strings.verifySignature );

			fetch( ethpressLoginWP.ajaxUrl, {
				method: 'POST',
				body: data
			})
			.then( function( res ) {
				if ( res.ok ) {
					return res.json();
				}
				throw new Error( res.statusText );
			})
			.then( function( result ) {
				if ( result.success ) {
					return result.data;
				}
				throw new Error( result.data.message );
			})
			.then( function( data ) {

				// Succesful login.
				setStatus( data.message );
				document.location.href = data.redirect;
			})
			.catch( function( reason ) {

				// Failed login. Enable button again.
				setStatus( reason, false );
			});
		}
	};

	return {
		metamask: metamask
	};
}( window.web3, window.ethereum, window.ethpressLoginWP ) );
